/**************************************************************************/
/*                                                                        */
/*  Thibaut Balabonski, Sylvain Conchon, Jean-Christophe Filliâtre,       */
/*  Kim Nguyen, Laurent Sartre                                            */
/*                                                                        */
/*  Informatique - MP2I/MPI - CPGE 1re et 2e années.                      */
/*  Cours et exercices corrigés. Éditions Ellipses, 2022.                 */
/*                                                                        */
/*  https://www.informatique-mpi.fr/                                      */
/*                                                                        */
/**************************************************************************/

#ifndef ARITH
#define ARITH

int arith_gcd(int u, int v);
  // pgcd de u et v

int arith_extended_gcd(int u, int v, int *x, int *y);
  // renvoie le pgcd de u et v, et les coefficients de Bezout dans *x et *y
  // on a gcd(u,v) = *x * u + *y * v avec |x|<=|v| et |y|<=|u|

int arith_power_naive(int x, int n);

int arith_power(int x, int n);
  // exponentiation rapide

uint64_t arith_power_mod(uint64_t x, uint64_t n, uint64_t m);
  // x^n modulo m

uint64_t arith_inv_mod(uint64_t x, uint64_t m);
  // inverse de x modulo m, en supposant gcd(x,m)=1

bool *arith_sieve(int max);
  // crible d'Ératosthène, jusqu'à max inclus
  // alloue et renvoie un tableau de booléens

int *arith_first_n_primes(int max);

bool arith_fermat(uint32_t n, int k);
  // test de primalité probabiliste :
  // - si la fonction renvoie `false` alors `n` est composé
  // - si la fonction renvoie `true`, alors `n` est premier avec une
  //   probabilité 1-1/2^k

bool arith_miller_rabin(uint32_t n, int k);
  // test de primalité probabiliste :
  // - si la fonction renvoie `false` alors `n` est composé
  // - si la fonction renvoie `true`, alors `n` est premier avec une
  //   probabilité 1-1/4^k

// exemple illustrant le renvoi d'une structure, contenant ici
// quotient et reste de la division Euclidienne
typedef struct Euclidean { int quotient, remainder; } euclidean;

euclidean arith_division(int a, int b);

#endif
